/***************************************************************************
                          calendar.h  -  description
                             -------------------
    begin                : Tue Feb 22 2000
    copyright            : (C) 2000 by the Genes Development Team
													 Sean Lamb, Project Lead
    email                : slambo@msn.fullfeed.com

  02/25/00	Added MAXNAMECHARS to help get the date strings the right length
  in GD_get_date_string
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

// Much of the information for setting up this file came from http://astro.nmsu.edu/~lhuber/leaphist.html

#ifndef CALENDAR_H
#define CALENDAR_H

#include <string>

// The total number of calendars
#define MAX_CALENDARS	  (7)
// The maximum number of months per calendar
#define MAX_MONTHS			(14)
// The maximum number of years per calendar
#define MAX_YEARS			  (12)
// The maximum number of characters in a month name
// Currently, the longest month name is Dhu al-Hijjah in the Islamic calendar
#define MAX_MNAMECHARS  (13)

// Define constants to identify data entry order...
#define MDY	(0)
#define DMY (1)
#define YMD (2)

// Define constants to identify the calendar...
// If you add a new calendar, add constants below for the months.
#define C_GREGORIAN		(0)
#define C_JULIAN			(1)
#define C_JEWISH			(2)
#define C_FRENCHREV		(3)
#define C_ISLAMIC			(4)
#define C_INDIAN			(5)
#define C_CHINESE			(6)

// Define constants to identify special days for calendars that support them...
// French Revoluionary calendar
#define D_JOUR_DE_LA_VERTU				(1)
#define D_JOUR_DU_GENIE           (2)
#define D_JOUR_DU_TRAVAIL         (3)
#define D_JOUR_DE_LOPINION        (4)
#define D_JOUR_DES_RECOMPENSES    (5)
#define D_JOUR_DE_LA_REVOLUTION   (6)

// Define constants to identify months...
// First, the Gregorian and Julian calendars
#define M_JANUARY			(1)
#define	M_FEBRUARY		(2)
#define M_MARCH				(3)
#define M_APRIL				(4)
#define M_MAY					(5)
#define M_JUNE				(6)
#define M_JULY				(7)
#define M_AUGUST			(8)
#define M_SEPTEMBER		(9)
#define M_OCTOBER			(10)
#define M_NOVEMBER		(11)
#define M_DECEMBER		(12)
// The Jewish calendar...
// Month names according to http://www.uwm.edu/~corre/calendar.html (2000-02-22)
#define M_NISAN				(1)
#define M_IYYAR				(2)
#define M_SIVAN				(3)
#define M_TAMMUZ			(4)
#define M_AV					(5)
#define M_ELUL				(6)
#define M_TISHRE			(7)
#define M_HESHVAN			(8)
#define M_KISLEV			(9)
#define M_TEVET				(10)
#define M_SHEVAT			(11)
#define M_ADAR				(12)
#define M_ADAR_II			(13)
// The French Revolutionary calendar...
#define M_VENDEMIAIRE		(1)
#define M_BRUMAIRE      (2)
#define M_FRIMAIRE      (3)
#define M_NIVOSE        (4)
#define M_PLUVIOSE      (5)
#define M_VENTOSE       (6)
#define M_GERMINCAL     (7)
#define M_FLOREAL       (8)
#define M_PRAIRIAL      (9)
#define M_MESSIDOR      (10)
#define M_THERMIDOR     (11)
#define M_FRUCTIDOR     (12)
// The Islamic calendar...
// From http://astro.nmsu.edu/~lhuber/leaphist.html (2000-02-22)
#define M_MUHARRAM			(1)
#define M_SAFAR					(2)
#define M_RABIA					(3)
#define M_RABIA_II			(4)
#define M_JUMADA				(5)
#define M_JUMADA_II			(6)
#define M_RAJAB					(7)
#define M_SHABAN				(8)
#define M_RAMADAN				(9)
#define M_SHAWWAL				(10)
#define M_DHU_AL_QADAH	(11)
#define M_DHU_AL_HIJJAH	(12)
// Indian from http://astro.nmsu.edu/~lhuber/leaphist.html (2000-02-22)
#define M_CAITRA				(1)
#define M_VAISAKHA      (2)
#define M_JYAISTHA      (3)
#define M_ASADHA        (4)
#define M_SRAVANA       (5)
#define M_BHADRA        (6)
#define M_ASVINA        (7)
#define M_KARTIKA       (8)
#define M_AGRAHAYANA    (9)
#define M_PAUSA         (10)
#define M_MAGHA         (11)
#define M_PHALGUNA      (12)
// Chinese
#define M_JIA						(1)
#define M_YI						(2)
#define M_BING          (3)
#define M_DING          (4)
#define M_WU            (5)
#define M_JI            (6)
#define M_GENG          (7)
#define M_XIN           (8)
#define M_REN           (9)
#define M_GUI           (10)

// Year names for calendars that support it...
// Chinese
#define Y_ZI							(1)
#define Y_CHOU            (2)
#define Y_YIN             (3)
#define Y_MAO             (4)
#define Y_CHEN            (5)
#define Y_SI              (6)
#define Y_WU              (7)
#define Y_WEI             (8)
#define Y_SHEN            (9)
#define Y_YOU             (10)
#define Y_XU              (11)
#define Y_HAI             (12)


const int month_lengths[MAX_CALENDARS][MAX_MONTHS] =
{
	/* GREGORIAN */ { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, 0 },
  /* JULIAN    */ { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, 0 },
  /* JEWISH    */ { 0, 30, 29, 30, 29, 30, 29, 30, 29, 30, 29, 30, 29, 0 },
  /* FRENCHREV */ { 0, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 5 },
  /* ISLAMIC   */ { 0, 30, 29, 30, 29, 30, 29, 30, 29, 30, 29, 30, 29, 0 },
  /* INDIAN    */ { 0, 30, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 30, 0 },
/* Need to find a reference that lists these month lengths */
  /* CHINESE   */ { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0 }  // 10 months
};

const string month_names[MAX_CALENDARS][MAX_MONTHS] =
{
	/* GREGORIAN */ { "", "January", "February", "March", "April", "May", "June",
										"July", "August", "September", "October", "November", "December", "" },
  /* JULIAN    */ { "", "January", "February", "March", "April", "May", "June",
										"July", "August", "September", "October", "November", "December", "" },
  /* JEWISH    */ { "", "Nisan", "Iyyar", "Sivan", "Tammuz", "Av", "Elul", "Tishre",
										"Heshvan", "Kislev", "Tevet", "Shevat", "Adar", "Adar-II" },
  /* FRENCHREV */ { "", "Vendemiaire", "Brumaire", "Frimaire", "Nivose", "Pluviose", "Ventose",
										"Germincal", "Floreal", "Prairial", "Messidor", "Thermidor", "Fructidor", "" },
  /* ISLAMIC   */ { "", "Muharram", "Safar", "Rabia", "Rabia-II", "Jumada", "Jumada-II",
										"Rajab", "Shaban", "Ramadan", "Shawwal", "Dhu al-Qadah", "Dhu al-Hijjah", "" },
  /* INDIAN    */ { "", "Caitra", "Vaisakha", "Jyaistha", "Asadha", "Sravana", "Bhadra",
										"Asvina", "Kartika", "Agrahayana", "Pausa", "Magha", "Phalguna", "" },
  /* CHINESE   */ { "",  "Jia", "Yi", "Bing", "Ding", "Wu", "Ji", "Geng", "Xin", "Ren", "Gui", "", "", "" }
};

const string month_names_abbr2[MAX_CALENDARS][MAX_MONTHS] =
{
	/* GREGORIAN */ { "", "JA", "FE", "MA", "AP", "MY", "JN", "JL", "AG", "SE", "OC", "NO", "DE", "" },
  /* JULIAN    */ { "", "JA", "FE", "MA", "AP", "MY", "JN", "JL", "AG", "SE", "OC", "NO", "DE", "" },
  /* JEWISH    */ { "", "NI", "IY", "SI", "TA", "AV", "EL", "TI",	"HE", "KI", "TE", "SH", "AD", "A2" },
  /* FRENCHREV */ { "", "VM", "BM", "FM", "NV", "PV", "VT",	"GL", "FL", "PL", "MD", "TD", "FD", "" },
  /* ISLAMIC   */ { "", "MU", "SA", "RB", "R2", "JM", "J2",	"RJ", "SB", "RM", "SW", "DQ", "DH", "" },
  /* INDIAN    */ { "", "CA", "VA", "JY", "AS", "SR", "BH",	"AV", "KA", "AG", "PA", "MA", "PH", "" },
  /* CHINESE   */ { "", "JA", "YI", "BN", "DN", "WU", "JI", "GN", "XN", "RN", "GI", "",    "",  "" }
};

const string month_names_abbr3[MAX_CALENDARS][MAX_MONTHS] =
{
	/* GREGORIAN */ { "", "JAN", "FEB", "MAR", "APR", "MAY", "JUN",	
										"JUL", "AUG", "SEP", "OCT", "NOV", "DEC", "" },
  /* JULIAN    */ { "", "JAN", "FEB", "MAR", "APR", "MAY", "JUN",	
										"JUL", "AUG", "SEP", "OCT", "NOV", "DEC", "" },
  /* JEWISH    */ { "", "NIS", "IYR", "SVN", "TMZ", "AV ", "ELL", "TSH",
										"HSH", "KSL", "TVT", "SVT", "ADR", "AD2" },
  /* FRENCHREV */ { "", "VMR", "BMR", "FMR", "NVS", "PLS", "VTS",
										"GML", "FLL", "PRL", "MDR", "TDR", "FDR", "" },
  /* ISLAMIC   */ { "", "MUH", "SFR", "RBA", "RB2", "JMD", "JM2",
										"RJB", "SBN", "RDN", "SWL", "DQD", "DHJ", "" },
  /* INDIAN    */ { "", "CTR", "VSK", "JYS", "ASD", "SVN", "BDR",
										"ASV", "KTK", "AGR", "PSA", "MGA", "PHA", "" },
  /* CHINESE   */ { "",  "JIA", "YI ", "BNG", "DNG", "WU ", "JI ",
										"GNG", "XIN", "REN", "GUI", "", "", "" }
};

const string frenchrev_days[7] =
{ "", "Jour de la Vertu", "Jour du Genie", "Jour du Travail", "Jour de L'Opinion",
	"Jour des Recompenses", "Jour de la Revolution" };

#endif
