/***************************************************************************
                          event.cpp  -  description
                             -------------------
    begin                : Fri Mar 17 2000
    copyright            : (C) 2000 by The Genes Development Team
    email                : slambo@msn.fullfeed.com
                           Geert.Vantienen@advalvas.be
    module               : Genes Datamodel
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "event.h"

// ------------------------------------------------------------------------
Event::Event(const GenesDataId& eid)
  : GenesData (eid),
    ExtraRecordData(),
    eventType_ (undefined),
    day_ (0),
    month_(0),
    year_(0),
    place_(""),
    cause_(""),
    originalDate_("")
{
}

// ------------------------------------------------------------------------
Event::Event(const GenesDataId& eid, EventType eventType)
  : GenesData (eid),
    ExtraRecordData(),
    eventType_ (eventType),
    day_ (0),
    month_(0),
    year_(0),
    place_(""),
    cause_(""),
    originalDate_("")
{
}

// ------------------------------------------------------------------------
// Event::Event(EventType eventType, int day, int month, int year, 
// 	     const String& place)
//   : ExtraRecordData(),
//     eventType_ (eventType),
//     day_ (day),
//     month_(month),
//     year_(year),
//     place_(place),
//     cause_(""),
//     originalDate_("")
// {
// }

// ------------------------------------------------------------------------
Event::~Event()
{
}

// ------------------------------------------------------------------------
Event::Event (const Event& event)
  : GenesData (event),
    ExtraRecordData(event),
    eventType_ (event.eventType_),
    day_ (event.day_),
    month_ (event.month_),
    year_ (event.year_),
    place_ (event.place_),
    cause_(event.cause_),
    originalDate_(event.originalDate_)
{
}

// ------------------------------------------------------------------------
void Event::setEventType (EventType eventType)
{
  eventType_ = eventType;
}

// ------------------------------------------------------------------------
void Event::setDate (int day, int month, int year) // @TODO also CalendarType
{
  day_ = day;
  month_ = month;
  year_ = year;
}

// ------------------------------------------------------------------------
void Event::setPlace (const String& place)
{
  place_ = place;
}

// ------------------------------------------------------------------------
void Event::setCause (const String& cause)
{
  cause_ = cause;
}

// ------------------------------------------------------------------------
Event::EventType Event::getEventType() const
{
  return (eventType_);
}

// ------------------------------------------------------------------------
int Event::getDay()  const
{
  return (day_);
}

// ------------------------------------------------------------------------
int Event::getMonth() const
{
  return (month_);
}

// ------------------------------------------------------------------------
int Event::getYear() const
{
  return (year_);
}

// ------------------------------------------------------------------------
const String& Event::getPlace() const
{
  return (place_);
}

// ------------------------------------------------------------------------
const String& Event::getCause() const
{
  return (cause_);
}

// ------------------------------------------------------------------------
const String& Event::getOriginalDateString() const
{
  return (originalDate_);
}

// ------------------------------------------------------------------------
bool Event::isValid() const
{
  return (getEventType() != Event::undefined);
}

// ------------------------------------------------------------------------
void Event::setOriginalDateString (char* origStr) // date as read from gedcom
{
  originalDate_ = origStr;
}

// ------------------------------------------------------------------------

