/***************************************************************************
                          event.h  -  description
                             -------------------
    begin                : Fri Mar 17 2000
    copyright            : (C) 2000 by The Genes Development Team
    email                : slambo@msn.fullfeed.com
                           Geert.Vantienen@advalvas.be
    module               : Genes Datamodel
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef EVENT_H
#define EVENT_H

/**
  *@author The Genes Development Team
  */

#include <datamodel/genesData.h>
#include <datamodel/extraRecordData.h>

class Event : public GenesData, public ExtraRecordData
{
public:
  typedef enum {undefined, birth, christening, baptism, bar_mitzvah,
		first_communion, graduation, naturalization, emigration, 
		immigration, census, engagement, marriage, 
		divorce, divorce_filed, 
		retirement, death, buriel, cremation, 
		other} EventType;

  Event(const GenesDataId&);
  Event(const GenesDataId&, EventType);
  ~Event();
  Event (const Event&);
  void setEventType (EventType);
  void setDate (int day, int month, int year); // @TODO also CalendarType
  void setOriginalDateString (char*); // date as read from gedcom
  void setPlace (const String&);
  void setCause (const String&);
  
  EventType     getEventType() const;
  int           getDay()  const;
  int           getMonth() const;
  int           getYear() const;
  const String& getPlace() const;
  const String& getCause() const;
  const String& getOriginalDateString() const;
  bool          isValid() const;

 private:
  EventType     eventType_;
  String        eventName_;    // only in case eventType==other
  int           day_;          // 0=unknown, 
  int           month_;
  int           year_;
  // later also calendarType
  // modifier for the date: <1900, or +/-1900, ... 
  // surity ?
  // ID of resource !

  String        originalDate_;
  String        place_;
  String        cause_;
};

#endif
