/***************************************************************************
                          family.cpp  -  description
                             -------------------
    begin                : Fri Mar 17 2000
    copyright            : (C) 2000 by The Genes Development Team
    email                : slambo@msn.fullfeed.com
                           Geert.Vantienen@advalvas.be
    module               : Genes Datamodel
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "family.h"

// ------------------------------------------------------------------------
Family::Family(const GenesDataId& id)
  : GenesData (id),
    ExtraRecordData(),
    fatherId_ (GenesDataId::personId),
    motherId_ (GenesDataId::personId),
    marriage_(0),
    divorce_(0)
{
}

// ------------------------------------------------------------------------
Family::~Family()
{
}

// ------------------------------------------------------------------------
Family::Family(const Family& fam)
  : GenesData (fam),
    ExtraRecordData(fam),
    fatherId_ (fam.fatherId_),
    motherId_ (fam.motherId_),
    marriage_ (fam.marriage_),
    divorce_ (fam.divorce_)
{
  // @TODO copy the lists ?
  cout << "@TODO copy constructor Family" << endl;
}

// ------------------------------------------------------------------------
void Family::setFather(const GenesDataId& id)
{
  fatherId_ = id;
}

// ------------------------------------------------------------------------
void Family::setMother(const GenesDataId& id)
{
  motherId_ = id;
}

// ------------------------------------------------------------------------
void Family::addChild(const GenesDataId& id)
{
  children_.push_back(id);
}

// ------------------------------------------------------------------------
const GenesDataId& Family::getFather() const
{
  return (fatherId_);
}

// ------------------------------------------------------------------------
const GenesDataId& Family::getMother() const
{
  return (motherId_);
}

// ------------------------------------------------------------------------
const list<GenesDataId>& Family::getChildren () const
{
  return (children_);
}

// ------------------------------------------------------------------------
Event* Family::getMarriageEvent ()
{
  return (marriage_);
}

// ------------------------------------------------------------------------
Event* Family::getDivorceEvent ()
{
  return (divorce_);
}

// ------------------------------------------------------------------------
void Family::addEvent (Event* event)
{
  if (event != 0)
    {
      switch (event->getEventType())
	{
	case Event::marriage : marriage_ = event; break;
	case Event::divorce : divorce_ = event; break;
	default : break; 
	}
      eventList_.push_back(event);
    }
}

// ------------------------------------------------------------------------
const list<Event*>& Family::getEventList () const
{
  return (eventList_);
}

// ------------------------------------------------------------------------
