/***************************************************************************
                         genesDataId.cpp
                        -------------------
    begin                : 10 Feb 2001
    copyright            : (C) 2001 by The Genes Development Team
    email                : Geert.Vantienen@advalvas.be
    module               : Genes Datamodel
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/
#include <datamodel/genesDataId.h>

// initialize static class members
int GenesDataId::lastPid_ = 0;
int GenesDataId::lastFid_ = 0;
int GenesDataId::lastEventId_ = 0;
int GenesDataId::lastAttributeId_ = 0;
int GenesDataId::lastSid_ = 0;
int GenesDataId::lastNid_ = 0;

// ------------------------------------------------------------------------
GenesDataId::GenesDataId ()
  : idType_ (unknown),
    id_ (-1),
    originalId_ ("")
{
}

// ------------------------------------------------------------------------
GenesDataId::GenesDataId (DataIdType idType)
  : idType_ (idType),
    id_ (-1),
    originalId_ ("")
{
}

// ------------------------------------------------------------------------
GenesDataId::GenesDataId (DataIdType idType, int id)
  : idType_ (idType),
    id_ (id),
    originalId_ ("")
{
}

// ------------------------------------------------------------------------
GenesDataId::GenesDataId (const GenesDataId& id)
  : idType_(id.idType_),
    id_(id.id_),
    originalId_ (id.originalId_)
{
}

// ------------------------------------------------------------------------
GenesDataId::~GenesDataId ()
{
}

// ------------------------------------------------------------------------
int GenesDataId::operator== (const GenesDataId& a) const
{
  int value = (idType_ == a.idType_);
  if (value)
    {
      value = (id_ == a.id_);
    }
  return (value);
}

// ------------------------------------------------------------------------
GenesDataId GenesDataId::getNextPersonId () // static
{
  // @TODO check for overflow ?
  return (GenesDataId (personId, ++lastPid_));
}
  
// ------------------------------------------------------------------------
GenesDataId GenesDataId::getNextFamilyId () // static
{
  // @TODO check for overflow ?
  return (GenesDataId (familyId, ++lastFid_));
}

// ------------------------------------------------------------------------
GenesDataId GenesDataId::getNextEventId () // static
{
  // @TODO check for overflow ?
  return (GenesDataId (eventId, ++lastEventId_));
}

// ------------------------------------------------------------------------
GenesDataId GenesDataId::getNextAttributeId () // static
{
  // @TODO check for overflow ?
  return (GenesDataId (attributeId, ++lastAttributeId_));
}

// ------------------------------------------------------------------------
GenesDataId GenesDataId::getNextSourceId () // static
{
  // @TODO check for overflow ?
  return (GenesDataId (sourceId, ++lastSid_));
}

// ------------------------------------------------------------------------
GenesDataId GenesDataId::getNextNoteId () // static
{
  // @TODO check for overflow ?
  return (GenesDataId (noteId, ++lastNid_));
}

// ------------------------------------------------------------------------
int GenesDataId::getId () const
{
  return (id_);
}
  
// ------------------------------------------------------------------------
bool GenesDataId::isPerson() const
{
  return (idType_ == personId);
}

// ------------------------------------------------------------------------
bool GenesDataId::isFamily() const
{
  return (idType_ == familyId);
}

// ------------------------------------------------------------------------
bool GenesDataId::isEvent() const
{
  return (idType_ == eventId);
}

// ------------------------------------------------------------------------
bool GenesDataId::isAttribute() const
{
  return (idType_ == attributeId);
}

// ------------------------------------------------------------------------
bool GenesDataId::isSource() const
{
  return (idType_ == sourceId);
}

// ------------------------------------------------------------------------
bool GenesDataId::isNote() const
{
  return (idType_ == noteId);
}

// ------------------------------------------------------------------------
int GenesDataId::isValid() const // @TODO return boolean
{
  return (id_ != -1);
}

// ------------------------------------------------------------------------
void GenesDataId::setOriginalId (const String& id)
{
  originalId_ = id;
}

// ------------------------------------------------------------------------
String GenesDataId::getOriginalId () const
{
  return (originalId_);
}

// ------------------------------------------------------------------------
