#ifndef GENESDATAID_H
#define GENESDATAID_H

/***************************************************************************
                           genesDataId.h
                        -------------------
    begin                : 10 Feb 2001
    copyright            : (C) 2001 by The Genes Development Team
    email                : Geert.Vantienen@advalvas.be
    module               : Genes Datamodel
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/
// This class represents the unique identifier for Person, Family, Resource, .
// The identifier cannot be the same as the gedcom identifier
// (for example when we want to merge two gedcom files)
// but (later) we can store the corresponding gedcom id in this class ...

#include <datamodel/String.h>

class GenesDataId
{
 public:
  typedef enum {personId, familyId, eventId, attributeId, sourceId, 
		noteId, unknown} DataIdType;

  GenesDataId ();
  GenesDataId (DataIdType);
  GenesDataId (DataIdType, int id);
  ~GenesDataId ();
  GenesDataId (const GenesDataId&);

  int operator== (const GenesDataId&) const;
  int                getId() const;

  static GenesDataId getNextPersonId ();
  static GenesDataId getNextFamilyId ();
  static GenesDataId getNextEventId ();
  static GenesDataId getNextAttributeId ();
  static GenesDataId getNextSourceId ();
  static GenesDataId getNextNoteId ();

  bool               isPerson() const;
  bool               isFamily() const;
  bool               isEvent() const;
  bool               isAttribute() const;
  bool               isSource() const;
  bool               isNote() const;

  int                isValid() const;
  void               setOriginalId (const String&);
  String             getOriginalId () const;

 private:
  DataIdType idType_;
  int        id_;
  String     originalId_;  // original IDstring coming from the loaded file

  static int lastPid_;
  static int lastFid_;
  static int lastEventId_;
  static int lastAttributeId_;
  static int lastSid_;
  static int lastNid_;
};

#endif
