/***************************************************************************
                         genesInfomodel.cpp
                        -------------------
    begin                : 16 Jul 2000
    copyright            : (C) 2000 by The Genes Development Team
    email                : Geert.Vantienen@advalvas.be
    module               : Genes Datamodel
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

// This class should be used to access all Information ...
// as well by userInterface module as other (import, export, wizard) modules
// need better name for this class ?
// to be used as singleton !

#include "genesInfomodel.h"
#include "personInfo.h"
#include "person.h"
#include "family.h"
#include "event.h"
#include "attribute.h"
#include "source.h"
#include "note.h"

// ------------------------------------------------------------------------
GenesInfomodel::GenesInfomodel()
{
}

// ------------------------------------------------------------------------
GenesInfomodel::~GenesInfomodel()
{
  // @TODO delete elements from all lists.
}

// ------------------------------------------------------------------------
void GenesInfomodel::addPerson (Person* person)    // need boolean for this ?
{
  personList_.push_back (person);
  notify(); // notify this update to its observers; @TODO improve
}

// ------------------------------------------------------------------------
void GenesInfomodel::addFamily (Family* fam)    // need boolean for this ?
{
  familyList_.push_back (fam);
}

// ------------------------------------------------------------------------
void GenesInfomodel::addEvent (Event* event)
{
  eventList_.push_back (event);
}

// ------------------------------------------------------------------------
void GenesInfomodel::addAttribute (Attribute* attrib)
{
  attributeList_.push_back (attrib);
}

// ------------------------------------------------------------------------
void GenesInfomodel::addSource (Source* src)
{
  sourceList_.push_back (src);
}

// ------------------------------------------------------------------------
void GenesInfomodel::addNote (Note* note)
{
  noteList_.push_back (note);
}

// ------------------------------------------------------------------------
void GenesInfomodel::buildInfoList()
{
  infolist_.clearList ();
  list<Person*>::iterator iter = personList_.begin();
  for (; iter != personList_.end(); ++iter)
    {
      Person* person = *iter;
      if (person != 0)
	{
	  int byear = 0;
	  int dyear = 0;
	  Event* birth = person->getBirthEvent();
	  if (birth != 0)
	    {
	      byear = birth->getYear();
	    }
	  Event* death = person->getDeathEvent();
	  if (death != 0)
	    {
	      dyear = death->getYear();
	    }
	  PersonInfo* info = new PersonInfo (person->getId(),0,
					     person->getSurname(),
					     person->getFirstname(),
					     byear,dyear);
	  //cout << "person Info, birth="<<byear<<" death="<<dyear<<endl;
	  // @TODO change byear and dyear to strings, and keep them this way!
	  infolist_.add (info);
	}
    }
}

// ------------------------------------------------------------------------
PersonInfoList& GenesInfomodel::getPersonInfoList () // should return const ?
{
  return (infolist_);
}

// ------------------------------------------------------------------------
Person* GenesInfomodel::getPerson (const GenesDataId& pid)
{
  Person* found = 0;
  int     loop  = 1;

  if ((! pid.isValid()) || (! pid.isPerson()))
    {
      loop = 0;
    }
  list<Person*>::iterator iter = personList_.begin();
  while (loop)
    {
      if (iter == personList_.end())
	{
	  loop = 0;
	}
      else
	{
	  Person* temp = *iter;
	  if (temp != 0)
	    {
	      if (temp->getId() == pid)
		{
		  found = temp;
		}
	    }
	  iter++;
	}
    }

 return (found);
}

// ------------------------------------------------------------------------
Family* GenesInfomodel::getFamily (const GenesDataId& fid)
{
  Family* found = 0;
  int     loop  = 1;

  if ((! fid.isValid()) || (! fid.isFamily()))
    {
      loop = 0;
    }
  list<Family*>::iterator iter = familyList_.begin();
  while (loop)
    {
      if (iter == familyList_.end())
	{
	  loop = 0;
	}
      else
	{
	  Family* temp = *iter;
	  if (temp != 0)
	    {
	      if (temp->getId() == fid)
		{
		  found = temp;
		}
	    }
	  iter++;
	}
    }

 return (found);
}

// ------------------------------------------------------------------------
Event* GenesInfomodel::getEvent (const GenesDataId& eid)
{
  Event* found = 0;
  int     loop  = 1;

  if ((! eid.isValid()) || (! eid.isEvent()))
    {
      loop = 0;
    }
  list<Event*>::iterator iter = eventList_.begin();
  while (loop)
    {
      if (iter == eventList_.end())
	{
	  loop = 0;
	}
      else
	{
	  Event* temp = *iter;
	  if (temp != 0)
	    {
	      if (temp->getId() == eid)
		{
		  found = temp;
		}
	    }
	  iter++;
	}
    }

 return (found);
}

// ------------------------------------------------------------------------
Attribute* GenesInfomodel::getAttribute (const GenesDataId& aid)
{
  Attribute* found = 0;
  int     loop  = 1;

  if ((! aid.isValid()) || (! aid.isAttribute()))
    {
      loop = 0;
    }
  list<Attribute*>::iterator iter = attributeList_.begin();
  while (loop)
    {
      if (iter == attributeList_.end())
	{
	  loop = 0;
	}
      else
	{
	  Attribute* temp = *iter;
	  if (temp != 0)
	    {
	      if (temp->getId() == aid)
		{
		  found = temp;
		}
	    }
	  iter++;
	}
    }

 return (found);
}

// ------------------------------------------------------------------------
Source* GenesInfomodel::getSource (const GenesDataId& sid)
{
  Source* found = 0;
  int     loop  = 1;

  if ((! sid.isValid()) || (! sid.isSource()))
    {
      loop = 0;
    }
  list<Source*>::iterator iter = sourceList_.begin();
  while (loop)
    {
      if (iter == sourceList_.end())
	{
	  loop = 0;
	}
      else
	{
	  Source* temp = *iter;
	  if (temp != 0)
	    {
	      if (temp->getId() == sid)
		{
		  found = temp;
		}
	    }
	  iter++;
	}
    }

 return (found);
}

// ------------------------------------------------------------------------
Note* GenesInfomodel::getNote (const GenesDataId& nid)
{
  Note* found = 0;
  int   loop  = 1;

  if ((! nid.isValid()) || (! nid.isNote()))
    {
      loop = 0;
    }
  list<Note*>::iterator iter = noteList_.begin();
  while (loop)
    {
      if (iter == noteList_.end())
	{
	  loop = 0;
	}
      else
	{
	  Note* temp = *iter;
	  if (temp != 0)
	    {
	      if (temp->getId() == nid)
		{
		  found = temp;
		}
	    }
	  iter++;
	}
    }

 return (found);
}

// ------------------------------------------------------------------------
PersonInfo* GenesInfomodel::getPersonInfo (const GenesDataId& pid)
{
  PersonInfo* person = 0;
  person = infolist_.getPersonInfo (pid);
  return (person);
}

// ------------------------------------------------------------------------
PersonInfo* GenesInfomodel::getFatherInfoFrom (const GenesDataId& pid)
{
  PersonInfo* father = 0;
  Person* person = getPerson (pid);
  if (person != 0)
    {
      const GenesDataId& fid = person->getChildFamily();
      Family* family = getFamily (fid);
      if (family != 0)
	{
	  const GenesDataId& fatherId = family->getFather();
	  father = infolist_.getPersonInfo (fatherId);
	}
    }
  return (father);
}

// ------------------------------------------------------------------------
PersonInfo* GenesInfomodel::getMotherInfoFrom (const GenesDataId& pid)
{
  PersonInfo* mother = 0;
  Person* person = getPerson (pid);
  if (person != 0)
    {
      const GenesDataId& fid = person->getChildFamily();
      Family* family = getFamily (fid);
      if (family != 0)
	{
	  const GenesDataId& motherId = family->getMother();
	  mother = infolist_.getPersonInfo (motherId);
	}
    }
  return (mother);
}

// ------------------------------------------------------------------------
void GenesInfomodel::clearList ()
{
  cout << "clearList (genesInfomodel)" << endl;
  list<Person*>::iterator iter1 = personList_.begin();
  while (iter1 != personList_.end())
    {
      Person* person = *iter1;
      delete person;
      iter1++;
    }
  while (! personList_.empty())
    {
      personList_.pop_front();
    }
  list<Family*>::iterator iter2 = familyList_.begin();
  while (iter2 != familyList_.end())
    {
      Family* family = *iter2;
      delete family;
      iter2++;
    }
  while (! familyList_.empty())
    {
      familyList_.pop_front();
    }
  list<Event*>::iterator iter3 = eventList_.begin();
  while (iter3 != eventList_.end())
    {
      Event* event = *iter3;
      delete event;
      iter3++;
    }
  while (! eventList_.empty())
    {
      eventList_.pop_front();
    }
  list<Attribute*>::iterator iter4 = attributeList_.begin();
  while (iter4 != attributeList_.end())
    {
      Attribute* attribute = *iter4;
      delete attribute;
      iter4++;
    }
  while (! attributeList_.empty())
    {
      attributeList_.pop_front();
    }
  list<Source*>::iterator iter5 = sourceList_.begin();
  while (iter5 != sourceList_.end())
    {
      Source* source = *iter5;
      delete source;
      iter5++;
    }
  while (! sourceList_.empty())
    {
      sourceList_.pop_front();
    }
  list<Note*>::iterator iter6 = noteList_.begin();
  while (iter6 != noteList_.end())
    {
      Note* note = *iter6;
      delete note;
      iter6++;
    }
  while (! noteList_.empty())
    {
      noteList_.pop_front();
    }
  infolist_.clearList();
}

// ------------------------------------------------------------------------
