/***************************************************************************
                          person.cpp  -  description
                             -------------------
    begin                : Fri Mar 17 2000
    copyright            : (C) 2000 by The Genes Development Team
    email                : slambo@msn.fullfeed.com
                           Geert.Vantienen@advalvas.be
    module               : Genes Datamodel
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

// this class keeps the information of a person

#include "person.h"
#include <datamodel/event.h>
#include <datamodel/attribute.h>

// ------------------------------------------------------------------------
Person::Person (const GenesDataId& pid)
  : GenesData (pid),
    ExtraRecordData (),
    firstname_ (""),
    surname_ (""),
    sex_(undefined),
    birth_(0),
    death_(0),
    childFamily_ (GenesDataId::familyId)
{
}

// ------------------------------------------------------------------------
Person::Person (const Person& info)
  : GenesData (info),
    ExtraRecordData (info),
    firstname_ (info.firstname_),
    surname_ (info.surname_),
    sex_(info.sex_),
    childFamily_(info.childFamily_),
    birth_ (info.birth_),
    death_ (info.death_)
{
  // is this really used ?
  // @TODO and how to copy the spouseFamily ?
  cout << "@TODO implement copy constructor Person" << endl;
}

// ------------------------------------------------------------------------
Person::~Person ()
{
}

// ------------------------------------------------------------------------
void Person::setFirstname(String name)
{
  firstname_ = name;
}

// ------------------------------------------------------------------------
void Person::setSurname(String name)
{
  surname_ = name;
}

// ------------------------------------------------------------------------
void Person::setChildFamily(const GenesDataId& fid)
{
  childFamily_ = fid;
}

// ------------------------------------------------------------------------
void Person::addSpouseFamily(const GenesDataId& fid)
{
  spouseFamily_.push_back(fid);
}

// ------------------------------------------------------------------------
// get the Name of the person
String Person::getFirstname () const
{
  return (firstname_);
}

// ------------------------------------------------------------------------
// get the Name of the person
String Person::getSurname () const
{
  return (surname_);
}

// ------------------------------------------------------------------------
const GenesDataId& Person::getChildFamily() const
{
  return (childFamily_);
}

// ------------------------------------------------------------------------
const list<GenesDataId>& Person::getSpouseFamily () const
{
  return (spouseFamily_);
}

// ------------------------------------------------------------------------
Event* Person::getBirthEvent ()
{
  return (birth_);
}

// ------------------------------------------------------------------------
Event* Person::getDeathEvent ()
{
  return (death_);
}

// ------------------------------------------------------------------------
void Person::addEvent (Event* event)
{
  if (event != 0)
    {
      switch (event->getEventType())
	{
	case Event::birth : birth_ = event; break;
	case Event::death : death_ = event; break;
	default : break;
	}
      eventList_.push_back(event);
    }
}

// ------------------------------------------------------------------------
const list<Event*>& Person::getEventList () const
{
  return (eventList_);
}

// ------------------------------------------------------------------------
void Person::addAttribute (Attribute* attribute)
{
  if (attribute != 0)
    {
      attributeList_.push_back(attribute);
    }
}

// ------------------------------------------------------------------------
const list<Attribute*>& Person::getAttributeList () const
{
  return (attributeList_);
}

// ------------------------------------------------------------------------
void Person::setSex (SexType sex)
{
  sex_ = sex;
}

// ------------------------------------------------------------------------
Person::SexType Person::getSex () const
{
  return (sex_);
}

// ------------------------------------------------------------------------

