/***************************************************************************
                          person.h  -  description
                             -------------------
    begin                : Fri Mar 17 2000
    copyright            : (C) 2000 by The Genes Development Team
    email                : slambo@msn.fullfeed.com
                           Geert.Vantienen@advalvas.be
    module               : Genes Datamodel
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef PERSON_H
#define PERSON_H


/**
  *@author The Genes Development Team
  */

#include <datamodel/genesData.h>
#include <datamodel/extraRecordData.h>
#include <stl.h>

class Event;
class Attribute;

class Person : public GenesData, public ExtraRecordData
{
 public:
  typedef enum {undefined, male, female, } SexType;

  Person (const GenesDataId&);
  ~Person ();
  Person (const Person&);

  void                setFirstname(String);
  void                setSurname(String);
  //void                setBirthyear(int);
  void                setChildFamily (const GenesDataId&); // better addChildF.
  void                addSpouseFamily (const GenesDataId&);

  String              getFirstname () const;
  String              getSurname () const;
  const GenesDataId&  getChildFamily() const;
  const list<GenesDataId>&  getSpouseFamily () const;

  void                addEvent (Event*);
  Event*              getBirthEvent();
  Event*              getDeathEvent();
  const list<Event*>& getEventList () const;
  void                addAttribute (Attribute*);
  const list<Attribute*>& getAttributeList () const;

  void                setSex (SexType);
  SexType             getSex () const;

 private:
  String              firstname_;
  String              surname_;
  GenesDataId         childFamily_; // @REMARK need a list for this, or not ?
  list<GenesDataId>   spouseFamily_;
  Event*              birth_;
  Event*              death_;
  SexType             sex_;
  list<Event*>        eventList_;
  list<Attribute*>    attributeList_;

    // A linked list of events, sorted by date...
    // A linked list of attributes
};

#endif
