/***************************************************************************
                          importGedcom.cpp
                        -------------------
    begin                : 30 Dec 2001
    copyright            : (C) 2001 by The Genes Development Team
    email                : Geert.Vantienen@advalvas.be
    module               : Genes gedcom
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "importGedcom.h"
#include <genesCore/genesApplication.h>
#include <datamodel/genesInfomodel.h>
#include <datamodel/family.h>
#include <datamodel/person.h>
#include <datamodel/note.h>
#include <datamodel/source.h>
#include <datamodel/attribute.h>
#include <modules.h>
#include <iostream.h>

#include <iconv.h>
#include <langinfo.h>
#include <errno.h>

// ------------------------------------------------------------------------
// initialize static class members
ImportGedcom* ImportGedcom::myself_ = 0;
MsgHandler ImportGedcom::handler_ = NULL;

// ------------------------------------------------------------------------
const char* getString (Gedcom_val parsed_value)
{
  const char* str1 = GEDCOM_STRING(parsed_value);
  return (str1);
}

// ------------------------------------------------------------------------
ImportGedcom::ImportGedcom (GenesApplication& applic)
  : GenesImportCore (applic, IMPORT_GEDCOM_MOD),
    datamodel_(0)
{
  if (myself_ != 0)
    {
      cout << "ERROR: ImportGedcom already instantiated !" << endl;
    }
  gedcom_init();
  myself_ = this;
}

// ------------------------------------------------------------------------
ImportGedcom::~ImportGedcom ()
{
}

// ------------------------------------------------------------------------
GenesDataId ImportGedcom::translateIdFrom (Gedcom_val parsed_value, 
					   GenesDataId::DataIdType idType)
{
  GenesDataId id;
  
  const char* key = 0;
  if (GEDCOM_XREF_PTR(parsed_value) != 0)
    {
      key = GEDCOM_XREF_PTR(parsed_value)->string;
    }
  else
    {
      key = "";
    }
  map<String, GenesDataId>::const_iterator iter = gedKeys_.find (key);
  if (iter == gedKeys_.end())
    {
      bool ok = true;
      switch (idType)
	{
	case GenesDataId::personId : 
	  id = GenesDataId::getNextPersonId();
	  gedKeys_[key] = id;
	  break;
	case GenesDataId::familyId : 
	  id = GenesDataId::getNextFamilyId();
	  gedKeys_[key] = id;
	  break;
	case GenesDataId::sourceId : 
	  id = GenesDataId::getNextSourceId();
	  gedKeys_[key] = id;
	  break;
	case GenesDataId::noteId : 
	  id = GenesDataId::getNextNoteId();
	  gedKeys_[key] = id;
	  break;
	default : 
	  cerr << "NOT supported dataID ... skipping it" << endl;
	  ok = false;
	}
      if (ok)
	{
	  id.setOriginalId(key);
	}
    }
  else
    {
      // @TODO check the expected dataId type ?
      id = gedKeys_[key];
    }
  return (id);
}

// ------------------------------------------------------------------------
typedef struct 
{
  Event* event;
  Person* person;
  Family* family;
  Source* source;
  Note*   note;
} Context_type;

Context_type* getNewContext()
{
  Context_type* ctxt = new Context_type;
  ctxt->event = 0;
  ctxt->person = 0;
  ctxt->family = 0;
  ctxt->source = 0;
  ctxt->note   = 0;
  return (ctxt);
}

Event* getContextEvent (Context_type* ctxt)
{
  Event* ev = 0;
  if (ctxt !=0) 
    ev = ctxt->event;
  return (ev);
}

Person* getContextPerson (Context_type* ctxt)
{
  Person* pers = 0;
  if (ctxt != 0)
    pers = ctxt->person;
  return (pers);
}

Family* getContextFamily (Context_type* ctxt)
{
  Family* fam = 0;
  if (ctxt != 0)
    fam = ctxt->family;
  return (fam);
}

Source* getContextSource (Context_type* ctxt)
{
  Source* src = 0;
  if (ctxt != 0)
    src = ctxt->source;
  return (src);
}

Note* getContextNote (Context_type* ctxt)
{
  Note* note = 0;
  if (ctxt != 0)
    note = ctxt->note;
  return (note);
}

void setContextEvent (Context_type* ctxt, Event* e)
{
  ctxt->event = e;
}

void setContextPerson (Context_type* ctxt, Person* p)
{
  ctxt->person = p;
}

void setContextFamily (Context_type* ctxt, Family* f)
{
  ctxt->family = f;
}

void setContextSource (Context_type* ctxt, Source* s)
{
  ctxt->source = s;
}

void setContextNote (Context_type* ctxt, Note* n)
{
  ctxt->note = n;
}

// ------------------------------------------------------------------------
Gedcom_ctxt person_start(int level, Gedcom_val xref, char *tag,
			 char *raw_value, int tag_value,
			 Gedcom_val parsed_value)
{
  const GenesDataId id 
    = ImportGedcom::myself_->translateIdFrom (xref, GenesDataId::personId);
  Context_type* ctxt = getNewContext();
  Person* pers = new Person (id);
  setContextPerson (ctxt, pers);
  ImportGedcom::myself_->datamodel_->addPerson (pers);
  return (Gedcom_ctxt)(ctxt);
}

// ------------------------------------------------------------------------
void person_end(Gedcom_ctxt self)
{
  // TODO delete the context ?
  //setContextPerson ((Context_type*)self, 0);
}

// ------------------------------------------------------------------------
Gedcom_ctxt family_start(int level, Gedcom_val xref, char *tag,
			 char *raw_value, int tag_value,
			 Gedcom_val parsed_value)
{
  const GenesDataId id 
    = ImportGedcom::myself_->translateIdFrom (xref, GenesDataId::familyId);
  Context_type* ctxt = getNewContext();
  Family* fam = new Family (id);
  setContextFamily (ctxt, fam);
  ImportGedcom::myself_->datamodel_->addFamily (fam);
  return (Gedcom_ctxt)(ctxt);
}

// ------------------------------------------------------------------------
void family_end(Gedcom_ctxt self)
{
  //setContextFamily ((Context_type*)self, 0);
}

// ------------------------------------------------------------------------
Gedcom_ctxt source_start(int level, Gedcom_val xref, char *tag,
			 char *raw_value, int tag_value,
			 Gedcom_val parsed_value)
{
  const GenesDataId id 
    = ImportGedcom::myself_->translateIdFrom (xref, GenesDataId::sourceId);
  Context_type* ctxt = getNewContext();
  Source* src = new Source (id);
  setContextSource (ctxt, src);
  ImportGedcom::myself_->datamodel_->addSource (src);
  return (Gedcom_ctxt)(ctxt);
}

// ------------------------------------------------------------------------
void source_end(Gedcom_ctxt self)
{
  //setContextSource ((Context_type*)self, 0);
}

// ------------------------------------------------------------------------
Gedcom_ctxt note_start(int level, Gedcom_val xref, char *tag,
		       char *raw_value, int tag_value,
		       Gedcom_val parsed_value)
{
  const GenesDataId id 
    = ImportGedcom::myself_->translateIdFrom (xref, GenesDataId::noteId);
  Context_type* ctxt = getNewContext();
  Note* note = new Note (id);
  setContextNote (ctxt, note);
  ImportGedcom::myself_->datamodel_->addNote (note);
  return (Gedcom_ctxt)(ctxt);
}

// ------------------------------------------------------------------------
void note_end(Gedcom_ctxt self)
{
  //setContextNote ((Context_type*)self, 0);
}

// ------------------------------------------------------------------------
Gedcom_ctxt indi_sex(Gedcom_ctxt parent, int level, char *tag,
		     char* raw_value, int tag_value, Gedcom_val parsed_value)
{
  Person* pers = getContextPerson ((Context_type*)parent);
  const char* sex = getString(parsed_value);
  if (sex[0] == 'F')
    {
      pers->setSex (Person::female);
    }
  else if (sex[0] == 'M')
    {
      pers->setSex (Person::male);
    }
  else
    {
      cout << "Parse error reading sexType" << endl;
    }
  return (Gedcom_ctxt)0;
}

// ------------------------------------------------------------------------
Gedcom_ctxt fam_husb(Gedcom_ctxt parent, int level, char *tag,
		     char* raw_value, int tag_value, Gedcom_val parsed_value)
{
  Family* fam = getContextFamily ((Context_type*)parent);
  const GenesDataId id 
    = ImportGedcom::myself_->translateIdFrom (parsed_value, 
					      GenesDataId::personId);
  fam->setFather (id);
  return (Gedcom_ctxt)parent;
}

// ------------------------------------------------------------------------
Gedcom_ctxt fam_wife(Gedcom_ctxt parent, int level, char *tag,
		     char* raw_value, int tag_value, Gedcom_val parsed_value)
{
  Family* fam = getContextFamily ((Context_type*)parent);
  const GenesDataId id 
    = ImportGedcom::myself_->translateIdFrom (parsed_value, 
					      GenesDataId::personId);
  fam->setMother (id);
  return (Gedcom_ctxt)parent;
}

// ------------------------------------------------------------------------
Gedcom_ctxt fam_chil(Gedcom_ctxt parent, int level, char *tag,
		     char* raw_value, int tag_value, Gedcom_val parsed_value)
{
  Family* fam = getContextFamily ((Context_type*)parent);
  const GenesDataId id 
    = ImportGedcom::myself_->translateIdFrom (parsed_value, 
					      GenesDataId::personId);
  if (fam != 0)
    {
      fam->addChild (id);
    }
  return (Gedcom_ctxt)parent;
}

// ------------------------------------------------------------------------
Gedcom_ctxt indi_famc(Gedcom_ctxt parent, int level, char *tag,
		      char* raw_value, int tag_value, Gedcom_val parsed_value)
{
  Person* pers = getContextPerson ((Context_type*)parent);
  const GenesDataId id 
    = ImportGedcom::myself_->translateIdFrom (parsed_value, 
					      GenesDataId::familyId);
  pers->setChildFamily (id); // should be addChildFamily ?
  return (Gedcom_ctxt)parent;
}

// ------------------------------------------------------------------------
Gedcom_ctxt indi_fams(Gedcom_ctxt parent, int level, char *tag,
		      char* raw_value, int tag_value, Gedcom_val parsed_value)
{
  Person* pers = getContextPerson ((Context_type*)parent);
  const GenesDataId id 
    = ImportGedcom::myself_->translateIdFrom (parsed_value, 
					      GenesDataId::familyId);
  pers->addSpouseFamily (id);
  return (Gedcom_ctxt)parent;
}

// ------------------------------------------------------------------------
void parseName (const char* line, char* fname, char* sname)
{
  char * ptr,*ptr2 = 0;
  int pos = 0;
  ptr = strchr (line,'/');
  if (ptr == 0)
    {
      pos = strlen (line);
      strncpy (fname,line,pos);
      fname[pos]='\0';
      strcpy (sname, "");
      return;
    }
  pos = (int) (ptr-line);
  strncpy (fname,line,pos);
  if (fname[pos-1]==' ')
    {
      pos--;
    }
  fname[pos]='\0';

  ptr2 = strchr (ptr+1,'/');
  if (ptr2 != 0)
    {
      strncpy (sname,ptr+1,ptr2-ptr-1);
      sname[ptr2-ptr-1] = '\0';
    }
  else                                   // not needed ???
    {
      strcpy (sname,ptr+1);
      if (sname[strlen(sname)-1] == '/')
	{
	  sname[strlen(sname)-1] = '\0';
	}
    }
}

// ------------------------------------------------------------------------
Gedcom_ctxt indi_name(Gedcom_ctxt parent, int level, char *tag,
		      char* raw_value, int tag_value, Gedcom_val parsed_value)
{
  char tmp1 [280];
  char tmp2 [280];
  Person* pers = getContextPerson ((Context_type*)parent);
  const char* name = getString(parsed_value);
  parseName (name, tmp1, tmp2);
  pers->setFirstname (tmp1);
  pers->setSurname (tmp2);
  return (Gedcom_ctxt)parent;
}

// ------------------------------------------------------------------------
Event* getNewEvent (Event::EventType evType)
{
  Event* event = new Event (GenesDataId::getNextEventId(), evType);
  ImportGedcom::myself_->datamodel_->addEvent (event);
  return (event);
}

// ------------------------------------------------------------------------
Gedcom_ctxt indi_event_start(Gedcom_ctxt parent, int level, char *tag,
			    char* raw_value, int tag_value,
			     Gedcom_val parsed_value)
{
  Event* event = 0;
  switch (tag_value)
    {
    case TAG_BIRT : event = getNewEvent (Event::birth); break;
    case TAG_CHR  : event = getNewEvent (Event::christening); break;
    case TAG_BAPM : event = getNewEvent (Event::baptism); break;
    case TAG_BARM : event = getNewEvent (Event::bar_mitzvah); break;
    case TAG_FCOM : event = getNewEvent (Event::first_communion); break;
    case TAG_GRAD : event = getNewEvent (Event::graduation); break;
    case TAG_NATU : event = getNewEvent (Event::naturalization); break;
    case TAG_EMIG : event = getNewEvent (Event::emigration); break;
    case TAG_IMMI : event = getNewEvent (Event::immigration); break;
    case TAG_CENS : event = getNewEvent (Event::census); break;
    case TAG_RETI : event = getNewEvent (Event::retirement); break;
    case TAG_DEAT : event = getNewEvent (Event::death); break;
    case TAG_BURI : event = getNewEvent (Event::buriel); break;
    case TAG_CREM : event = getNewEvent (Event::cremation); break;
    default : cout << "**** Unhandled event type: tag=" << tag << endl;
    }
  Context_type* ctxt = getNewContext();
  setContextEvent (ctxt, event);
  return (Gedcom_ctxt)ctxt;
}

// ------------------------------------------------------------------------
void indi_event_end(Gedcom_ctxt parent, Gedcom_ctxt self, Gedcom_val parsed_value)
{
  Person* pers = getContextPerson ((Context_type*)parent);
  Event* event = getContextEvent ((Context_type*)self);
  if ((pers !=0) && (event != 0))
    {
      pers->addEvent (event);
    }
  // @TODO delete this context object !
  setContextEvent ((Context_type*)self, 0);
}

// ------------------------------------------------------------------------
Gedcom_ctxt indi_attrib (Gedcom_ctxt parent, int level, char *tag,
			 char* raw_value, int tag_value,
			 Gedcom_val parsed_value)
{
  Attribute::AttributeType attrType = Attribute::undefined;
  switch (tag_value)
    {
    case TAG_EDUC : attrType = Attribute::education; break;
    case TAG_OCCU : attrType = Attribute::occupation; break;
    case TAG_RELI : attrType = Attribute::religion; break;
    case TAG_TITL : attrType = Attribute::title; break;
    case TAG_CAST : attrType = Attribute::caste; break;
    case TAG_DSCR : attrType = Attribute::phy_descr; break;
    case TAG_IDNO : attrType = Attribute::ident_no; break;
    case TAG_NATI : attrType = Attribute::nationality; break;
    case TAG_NCHI : attrType = Attribute::child_count; break;
    case TAG_NMR  : attrType = Attribute::marriage_count; break;
    case TAG_PROP : attrType = Attribute::property; break;
    case TAG_SSN  : attrType = Attribute::soc_sec; break;
    default : cout << "**** Unhandled attribure: tag=" << tag << endl;
    }
  Person* pers = getContextPerson ((Context_type*)parent);
  if (pers != 0)
    {
      Attribute* attrib = new Attribute (GenesDataId::getNextAttributeId(),
					 attrType, getString(parsed_value));
      ImportGedcom::myself_->datamodel_->addAttribute (attrib);
      pers->addAttribute (attrib);
    }
  return (parent);
}

// ------------------------------------------------------------------------
Gedcom_ctxt fam_event_start(Gedcom_ctxt parent, int level, char *tag,
			    char* raw_value, int tag_value,
			    Gedcom_val parsed_value)
{
  Event* event = 0;
  switch (tag_value)
    {
    case TAG_ENGA : event = getNewEvent (Event::engagement); break;
    case TAG_MARR : event = getNewEvent (Event::marriage); break;
    case TAG_DIV : event  = getNewEvent (Event::divorce); break;
    case TAG_DIVF : event = getNewEvent (Event::divorce_filed); break;
    default : cout << "**** Unhandled event type: tag=" << tag << endl;
    }
  Context_type* ctxt = getNewContext();
  setContextEvent (ctxt, event);
  return (Gedcom_ctxt)ctxt;
}

// ------------------------------------------------------------------------
void fam_event_end(Gedcom_ctxt parent, Gedcom_ctxt self, Gedcom_val parsed_value)
{
  Family* fam  = getContextFamily ((Context_type*)parent);
  Event* event = getContextEvent ((Context_type*)self);
  if ((fam !=0) && (event != 0))
    {
      fam->addEvent (event);
      // @TODO delete this event object ???
    }
  // @TODO delete this context object !
  setContextEvent ((Context_type*)self, 0);
}

// ------------------------------------------------------------------------
Gedcom_ctxt event_date (Gedcom_ctxt parent, int level, char *tag,
			char* raw_value, int tag_value, 
			Gedcom_val parsed_value)
{
  Event* event = getContextEvent ((Context_type*)parent);
  if (event != 0)
    {
      struct date_value dv;
      dv = GEDCOM_DATE(parsed_value);
      event->setDate (dv.date1.day,dv.date1.month,dv.date1.year);
      event->setOriginalDateString (raw_value);
    }
  else
    {
      cout << "== DATE not handled " << endl;
    }
  return ((Context_type*)parent);
}

// ------------------------------------------------------------------------
Gedcom_ctxt event_place (Gedcom_ctxt parent, int level, char *tag,
			 char* raw_value, int tag_value,
			 Gedcom_val parsed_value)
{
  Event* event = getContextEvent ((Context_type*)parent);
  if (event != 0)
    {
      event->setPlace (getString(parsed_value));
    }
  else
    {
      cout << "== PLACE not handled " << endl;
    }
  return ((Context_type*)parent);
}

// ------------------------------------------------------------------------
Gedcom_ctxt event_cause (Gedcom_ctxt parent, int level, char *tag,
			 char* raw_value, int tag_value, 
			 Gedcom_val parsed_value)
{
  Event* event = getContextEvent ((Context_type*)parent);
  if (event != 0)
    {
      event->setCause (getString(parsed_value));
    }
  else
    {
      cout << "== CAUSE not handled " << endl;
    }
  return ((Context_type*)parent);
}

// ------------------------------------------------------------------------
Gedcom_ctxt sub_note (Gedcom_ctxt parent, int level, char *tag,
		      char* raw_value, int tag_value, Gedcom_val parsed_value)
{
  // This has to change completely (change needed in datamodel !)
  ExtraRecordData* rec = getContextEvent ((Context_type*)parent);
  if (rec == 0)
    {
      rec = getContextPerson ((Context_type*)parent);
    }
  if (rec == 0)
    {
      rec = getContextFamily ((Context_type*)parent);
    }
  
  if (rec != 0)
    {
      if (GEDCOM_IS_XREF_PTR (parsed_value))
	{
	  const GenesDataId id 
	    = ImportGedcom::myself_->translateIdFrom (parsed_value, 
						      GenesDataId::sourceId);
	  NoteCitation citation (id);
	  rec->addNote (citation);
	}
      else
	{
	  NoteCitation citation (getString(parsed_value));
	  rec->addNote (citation);
	}
    }
  else
    {
      cout << "== NOTE not handled " << raw_value << endl;
    }
  return (parent);
}

// ------------------------------------------------------------------------
Gedcom_ctxt sub_source (Gedcom_ctxt parent, int level, char *tag,
			char* raw_value, int tag_value, 
			Gedcom_val parsed_value)
{
  // This has to change completely (change needed in datamodel !)
  ExtraRecordData* rec = getContextEvent ((Context_type*)parent);
  if (rec == 0)
    {
      rec = getContextPerson ((Context_type*)parent);
    }
  if (rec == 0)
    {
      rec = getContextFamily ((Context_type*)parent);
    }
  if (rec != 0)
    {
      if (GEDCOM_IS_XREF_PTR (parsed_value))
	{
	  const GenesDataId id 
	    = ImportGedcom::myself_->translateIdFrom (parsed_value, 
						      GenesDataId::sourceId);
	  SourceCitation citation (id);
	  rec->addSource (citation);
	}
      else
	{
	  SourceCitation citation (getString(parsed_value));
	  rec->addSource (citation);
	}
    }
  else
    {
      cout << "== SOURCE not handled " << raw_value << endl;
    }
  return (parent);
}

// ------------------------------------------------------------------------
Gedcom_ctxt source_rec_tag (Gedcom_ctxt parent, int level, char *tag,
			    char* raw_value, int tag_value, 
			    Gedcom_val parsed_value)
{
  Source* src = getContextSource ((Context_type*)parent);
  if (src != 0)
    {
      const char* str = getString(parsed_value);
      switch (tag_value)
	{
	case TAG_TITL : src->setTitle (str); break;
	case TAG_AUTH : src->setAuthor (str); break;
	case TAG_PUBL : src->setPublication (str); break;
	case TAG_TEXT : src->addText (str); break;
	default : cout << "*** unhandled SOURCE record tag" << endl;
	}
    }
  else
    {
      cout << "== SOURCE rec tags something wrong " << endl;
    }
  return (parent);
}

// ------------------------------------------------------------------------
Gedcom_ctxt default_rec(int level, Gedcom_val xref, char *tag,
			char *raw_value, int tag_value,
			Gedcom_val parsed_value)
{
  // to avoid problems : resetting context for not handled records
  return (Gedcom_ctxt)0;
}

// ------------------------------------------------------------------------
void default_cb(Gedcom_ctxt ctxt, int level, char *tag, char *raw_value,
		int tag_value)
{
//   cout << "== " << level << " " << tag << " ("<< tag_value << ") ";
//   cout << raw_value << "(ctxt is " << (int)ctxt << ")" << endl;
}

// ------------------------------------------------------------------------
void gedcom_message_handler(Gedcom_msg_type type, char *msg)
{
  if (ImportGedcom::handler_) {
    string s;
    if (type == MESSAGE)
      s = "MESSAGE: ";
    else if (type == WARNING)
      s = "WARNING: ";
    else if (type == ERROR)
      s = "ERROR: ";
    s += msg;
    s += "\n";
    (*ImportGedcom::handler_)(s);
  }
}

// ------------------------------------------------------------------------
void subscribe_callbacks()
{
  gedcom_subscribe_to_record(REC_INDI, person_start, person_end);
  gedcom_subscribe_to_record(REC_FAM, family_start, family_end);
  gedcom_subscribe_to_record(REC_HEAD, default_rec, 0);
  gedcom_subscribe_to_record(REC_OBJE, default_rec, 0);
  gedcom_subscribe_to_record(REC_NOTE, note_start, note_end);
  gedcom_subscribe_to_record(REC_REPO, default_rec, 0);
  gedcom_subscribe_to_record(REC_SOUR, source_start, source_end);
  gedcom_subscribe_to_record(REC_SUBN, default_rec, 0);
  gedcom_subscribe_to_record(REC_SUBM, default_rec, 0);
  gedcom_subscribe_to_record(REC_USER, default_rec, 0);


  gedcom_subscribe_to_element(ELT_INDI_SEX, indi_sex, 0);
  gedcom_subscribe_to_element(ELT_FAM_HUSB, fam_husb, 0);
  gedcom_subscribe_to_element(ELT_FAM_WIFE, fam_wife, 0);
  gedcom_subscribe_to_element(ELT_FAM_CHIL, fam_chil, 0);
  gedcom_subscribe_to_element(ELT_SUB_FAMC, indi_famc, 0);
  gedcom_subscribe_to_element(ELT_SUB_FAMS, indi_fams, 0);
  gedcom_subscribe_to_element(ELT_SUB_PERS_NAME, indi_name, 0);
  gedcom_subscribe_to_element(ELT_SUB_INDIV_GEN, indi_event_start, 
  			      indi_event_end);
  gedcom_subscribe_to_element(ELT_SUB_INDIV_BIRT, indi_event_start, 
			      indi_event_end);
  gedcom_subscribe_to_element(ELT_SUB_INDIV_ATTR, indi_attrib, 0);
  gedcom_subscribe_to_element(ELT_SUB_FAM_EVT, fam_event_start,
			      fam_event_end);
  gedcom_subscribe_to_element(ELT_SUB_EVT_DATE, event_date, 0); 
  gedcom_subscribe_to_element(ELT_SUB_PLAC, event_place, 0);
  gedcom_subscribe_to_element(ELT_SUB_EVT_CAUS, event_cause, 0);
  gedcom_subscribe_to_element(ELT_SUB_NOTE, sub_note, 0);
  gedcom_subscribe_to_element(ELT_SUB_SOUR, sub_source, 0);
  gedcom_subscribe_to_element(ELT_SOUR_TITL, source_rec_tag, 0);
  gedcom_subscribe_to_element(ELT_SOUR_AUTH, source_rec_tag, 0);
  gedcom_subscribe_to_element(ELT_SOUR_PUBL, source_rec_tag, 0);
  gedcom_subscribe_to_element(ELT_SOUR_TEXT, source_rec_tag, 0);
}

// ------------------------------------------------------------------------
// @TODO later return GenesStatus
void ImportGedcom::load (const String& filename, GenesInfomodel& datamodel,
			 MsgHandler hnd)
{
  datamodel_ = &datamodel;
  handler_ = hnd;
  cout << "Status: loading file ..." << endl;         // @TODO use showStatus

  //@TODO clean the translation map first !
  int debug_level = 0;
  int compat_enabled = 1;
  gedcom_set_debug_level(debug_level, NULL);
  gedcom_set_compat_handling(compat_enabled);
  gedcom_set_error_handling(IGNORE_ERRORS);
  gedcom_set_message_handler(gedcom_message_handler);
  gedcom_set_default_callback(default_cb);
  subscribe_callbacks();
  char* filenamestr = (char*) filename.c_str();
  int result = gedcom_parse_file(filenamestr);

  // @TODO notify ui that loading is done ...
  datamodel.buildInfoList();
  datamodel_ = 0;
}


// ------------------------------------------------------------------------
