/***************************************************************************
                           glade2UI.cpp
                        -------------------
    begin                : 14 Jan 2001
    copyright            : (C) 2000 by The Genes Development Team
    email                : Geert.Vantienen@advalvas.be
    module               : Genes glade2
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/
// a glade2 User Interface for Genes

#include "glade2UI.h"
#include "gladeUtil.h"
#include "GenesmainWnd.hh"
#include "PersonWnd.hh"
#include "FamilyWnd.hh"
#include "ImportWnd.hh"
#include "EventWnd.hh"

#include <genesCore/genesApplication.h>
#include <genesCore/genesImportCore.h>
#include <datamodel/family.h>
#include <datamodel/person.h>
#include <datamodel/personInfo.h>
#include <datamodel/personInfoList.h>
#include <datamodel/genesInfomodel.h>
#include <utils/showUtils.h>
#include <modules.h>
#include <config.h>
#include <iostream.h>
#include <stdio.h>
#include <gtk--/main.h>

ImportWnd* Glade2UI::myimport_ = NULL;

// ------------------------------------------------------------------------
Glade2UI::Glade2UI (GenesApplication& applic)
  : GenesUICore(applic, GUI_GLADE2_MOD),
    selectedPersonInfo_(0),
    selectedPid_(GenesDataId::personId),
    fatherPid_(GenesDataId::personId),
    motherPid_(GenesDataId::personId),
    spousePid_(GenesDataId::personId),
    childPid_(GenesDataId::personId),
    genesWnd_(0),
    mymain_(0)
{
}

// ------------------------------------------------------------------------
Glade2UI::~Glade2UI ()
{
  if (myimport_)
    delete myimport_;
  // @TODO delete genesmainWnd_ ???
}

// ------------------------------------------------------------------------
void Glade2UI::load ()
{
  if (!myimport_) {
    myimport_ = new ImportWnd(*this);
  }
  else {
    myimport_->show();
  }
  //loadTest();
}

// ------------------------------------------------------------------------
void Glade2UI::view ()
{
  showPerson();
}

// ------------------------------------------------------------------------
void Glade2UI::about ()
{
  cout << "This is " << PACKAGE << " " << VERSION << endl;
}

// ------------------------------------------------------------------------
void Glade2UI::selectPerson (const PersonInfo* person)
{
  if (person != 0)
    {
      int row = person->getRow();
      genesWnd_->selectRow(row);
      selectedPid_ = person->getPid();
      doOnSelect (row, 1); // @TODO improve this
    }
}

// ------------------------------------------------------------------------
void Glade2UI::selectFather ()
{
  if (fatherPid_.isValid())
    {
      PersonInfo* father = getFatherInfoFrom (selectedPid_);
      selectPerson (father);
    }
}

// ------------------------------------------------------------------------
void Glade2UI::selectMother ()
{
  if (motherPid_.isValid())
    {
      PersonInfo* mother = getMotherInfoFrom (selectedPid_);
      selectPerson (mother);
    }
}

// ------------------------------------------------------------------------
void Glade2UI::selectSpouse (const String& spouce)
{
  const GenesDataId* id = spousesInfo_.getIdFor (spouce);
  if (id != 0)
    {
      PersonInfo* spouse = getPersonInfo (*id);
      selectPerson (spouse);
    }
}

// ------------------------------------------------------------------------
void Glade2UI::selectChild (const String& child)
{
  const GenesDataId* id = childrenInfo_.getIdFor (child);
  if (id != 0)
    {
      PersonInfo* child = getPersonInfo (*id);
      selectPerson (child);
    }
}

// ------------------------------------------------------------------------
void Glade2UI::doOnSelect(int row, int col)
{
  cout << "Selected: " << row << "," << col << endl;
  //showStatus ("Selected: " + row + "," + col);
  if (genesWnd_ != 0)
    {
      gpointer data = genesWnd_->getRowData (row);
      PersonInfo* info = (PersonInfo*) data;
      if (info != 0)
	{
	  clearEntries();
	  selectedPersonInfo_ = info;
	  selectedPid_ = info->getPid ();
 	  cout << " PID = " << selectedPid_.getId() << endl;

	  Person* person = getPerson (selectedPid_);
	  if (person != 0)
	    {
	      String name = ShowUtils::getFullName (*person);
	      genesWnd_->setPerson (name.c_str());
	      
	      PersonInfo* father = getFatherInfoFrom (selectedPid_);
	      if (father != 0)
		{
		  String name2 = ShowUtils::getFullName2 (*father);
		  genesWnd_->setFather (name2.c_str());
		  fatherPid_ = father->getPid();
		}
	      
	      PersonInfo* mother = getMotherInfoFrom (selectedPid_);
	      if (mother != 0)
		{
		  String name3 = ShowUtils::getFullName2 (*mother);
		  genesWnd_->setMother (name3.c_str());
		  motherPid_ = mother->getPid();
		}
	      const list<GenesDataId>& fidList = person->getSpouseFamily();
	      int len = fidList.size(); // ???
	      cout << "spouses: " << len << endl;
	      list<GenesDataId>::const_iterator iter = fidList.begin();
	      list<string> gl;
	      list<string> childrenList;
	      //int first = 1;
	      while (iter != fidList.end())
		{
		  GenesDataId fid = *iter;
		  Family* fam = getFamily (fid);
		  if (fam != 0)
		    {
		      GenesDataId pid1 = fam->getFather();
		      GenesDataId pid2 = fam->getMother();
		      GenesDataId pid (GenesDataId::personId);
		      if (pid1 == selectedPid_)
			{
			  pid = pid2;
			}
		      else
			{
			  pid = pid1;
			}
		      PersonInfo* info = getPersonInfo (pid);
		      if (info != 0)
			{
			  String name3 = ShowUtils::getFullName2 (*info);
			  gl.push_back(name3);
			  spousesInfo_.addEntry (name3, pid);
			}
		      addChildrenTo (*fam, childrenList);
		    }
		  iter++;
		}
	      genesWnd_->setChildren (childrenList);
	      genesWnd_->setSpouses (gl);
	    }
	  if (col == 0)
	    {
	      int mark = ! (info->getMark());
	      info->setMark (mark);
	      genesWnd_->setMark (row, mark);
	    }
	} // END if (info != 0)
    }
}

// ------------------------------------------------------------------------
void Glade2UI::doOnDeselect(int row, int col)
{
  cout << "Deselect " << row << "," << col << endl;
//   if ((selectedPersonInfo_ != 0) && (col == 0))
//     {
//       int mark = ! (selectedPersonInfo_->getMark());
//       selectedPersonInfo_->setMark (mark);
//       genesWnd_->setMark (row, mark);
//     }
  selectedPersonInfo_ = 0;
  clearEntries();
}

// ------------------------------------------------------------------------
void Glade2UI::gtkinit (int argc, char* argv[])
{
  mymain_ = new Gtk::Main (&argc, &argv);
}

// ------------------------------------------------------------------------
void Glade2UI::initObject ()
{
  genesWnd_ = new GenesmainWnd(*this);
  GenesUICore::initObject(); // future proof ...
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  //datamodel->detach (this); // if already observing ...
  datamodel->attach (this);
}

// ------------------------------------------------------------------------
void Glade2UI::loop ()
{
  if (mymain_ != 0)
    {
      mymain_->run();
    }
  else
    {
      showError ("error in loop");
    }
}

// ------------------------------------------------------------------------
void Glade2UI::quit ()
{
  Gtk::Main::quit();
}

// ------------------------------------------------------------------------
void Glade2UI::showError (String text)
{
  // @TODO popup error box, and show message, close on Ok button
  cout << "ERROR : " << text << endl;
}

// ------------------------------------------------------------------------
void Glade2UI::showStatus (String text)
{
  cout << text << endl;
  if (genesWnd_ != 0)
    {
      genesWnd_->statusbar->push (1,text);
    }
}

// ------------------------------------------------------------------------
void Glade2UI::showEvent (Event* event, const String& name)
{
  GenesInfomodel* datamodel = getApplication().getDatamodel ();
  if ((datamodel != 0) && (event != 0))
    {
      EventWnd* eventWnd = new EventWnd(*this, *datamodel, *event, name, true);
      eventWnd->initObject();
    }
}

// ------------------------------------------------------------------------
void Glade2UI::showFamily (Family& family)
{
  GenesInfomodel* datamodel = getApplication().getDatamodel ();
  if (datamodel != 0)
    {
      FamilyWnd* familyWnd = new FamilyWnd(*this, *datamodel, family);
      familyWnd->initObject();
    }
}

// ------------------------------------------------------------------------
void Glade2UI::showPerson (Person& person)
{
  GenesInfomodel* datamodel = getApplication().getDatamodel ();
  if (datamodel != 0)
    {
      PersonWnd* personWnd = new PersonWnd(*this, *datamodel,  person, 
					   PersonWnd::showOp);
      personWnd->initObject();
    }
}

// ------------------------------------------------------------------------
void Glade2UI::showPerson (const GenesDataId& pid)
{
  Person* person = getPerson(pid);
  if (person != 0)
    {
      showPerson (*person);
    }
}

// ------------------------------------------------------------------------
void Glade2UI::showPerson ()
{
  showPerson (selectedPid_);
}

// ------------------------------------------------------------------------
void Glade2UI::showFather ()
{
  showPerson(fatherPid_);
}

// ------------------------------------------------------------------------
void Glade2UI::showMother ()
{
  showPerson(motherPid_);
}

// ------------------------------------------------------------------------
void Glade2UI::showSpouse (const String& spouce)
{
  const GenesDataId* id = spousesInfo_.getIdFor (spouce);
  if (id != 0)
    {
      showPerson (*id);
    }
}

// ------------------------------------------------------------------------
void Glade2UI::showChild (const String& child)
{
  const GenesDataId* id = childrenInfo_.getIdFor (child);
  if (id != 0)
    {
      showPerson (*id);
    }
}

// ------------------------------------------------------------------------
void Glade2UI::addPerson ()
{
  GenesInfomodel* datamodel = getApplication().getDatamodel ();
  if (datamodel != 0)
    {
      const GenesDataId& id = GenesDataId::getNextPersonId();
      Person* person = new Person (id);
      PersonWnd* personWnd = new PersonWnd(*this, *datamodel, *person, 
					   PersonWnd::newOp);
      personWnd->initObject();
      // @TODO who will cleanup this new person object ?
    }
}

// ------------------------------------------------------------------------
void Glade2UI::editEvent (Event& event, const String& name)
{
  GenesInfomodel* datamodel = getApplication().getDatamodel ();
  if (datamodel != 0)
    {
      EventWnd* eventWnd = new EventWnd(*this, *datamodel, event, name, false);
      eventWnd->initObject();
    }
}

// ------------------------------------------------------------------------
void Glade2UI::clearEntries()
{
  genesWnd_->setPerson ("");
  genesWnd_->setFather ("");
  genesWnd_->setMother ("");
  selectedPid_ = GenesDataId (GenesDataId::personId);
  fatherPid_ = GenesDataId (GenesDataId::personId);
  motherPid_ = GenesDataId (GenesDataId::personId);
  spousePid_ = GenesDataId (GenesDataId::personId);
  childPid_  = GenesDataId (GenesDataId::personId);
  spousesInfo_.clearList();
  childrenInfo_.clearList();
  list<string> gl;
  genesWnd_->setSpouses (gl);
  genesWnd_->setChildren (gl);
}

// ------------------------------------------------------------------------
void Glade2UI::addChildrenTo(Family& fam, list<string>& gl)
{
  const list<GenesDataId>& pidList = fam.getChildren();
  //int len = pidList.size();
  list<GenesDataId>::const_iterator iter = pidList.begin();
  
  while (iter != pidList.end())
    {
      GenesDataId pid = *iter;
      PersonInfo* info = getPersonInfo (pid);
      if (info != 0)
	{
	  String name = ShowUtils::getFullName2 (*info);
	  gl.push_back(name);
	  childrenInfo_.addEntry (name, pid);
	}
      iter++;
    }
}

// ------------------------------------------------------------------------
Person* Glade2UI::getPerson (const GenesDataId& pid)
{
  Person* person = 0;
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if ((pid.isValid()) && (datamodel != 0))
    {
      person = datamodel->getPerson (pid);
    }
  return (person);
}

// ------------------------------------------------------------------------
PersonInfo* Glade2UI::getPersonInfo (const GenesDataId& pid)
{
  PersonInfo* personInfo = 0;
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if ((pid.isValid()) && (datamodel != 0))
    {
      personInfo = datamodel->getPersonInfo (pid);
    }
  return (personInfo);
}

// ------------------------------------------------------------------------
PersonInfo* Glade2UI::getFatherInfoFrom (const GenesDataId& pid)
{
  PersonInfo* personInfo = 0;
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if ((pid.isValid()) && (datamodel != 0))
    {
      personInfo = datamodel->getFatherInfoFrom (pid);
    }
  return (personInfo);
}

// ------------------------------------------------------------------------
PersonInfo* Glade2UI::getMotherInfoFrom (const GenesDataId& pid)
{
  PersonInfo* personInfo = 0;
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if ((pid.isValid()) && (datamodel != 0))
    {
      personInfo = datamodel->getMotherInfoFrom (pid);
    }
  return (personInfo);
}

// ------------------------------------------------------------------------
Family* Glade2UI::getFamily (const GenesDataId& fid)
{
  Family* family = 0;
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if ((fid.isValid()) && (datamodel != 0))
    {
      family = datamodel->getFamily (fid);
    }
  return (family);
}

// ------------------------------------------------------------------------
GenesInfomodel* Glade2UI::getDatamodel ()
{
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  return (datamodel);
}

// ------------------------------------------------------------------------
void Glade2UI::loadTest()
{
  GenesImportCore* module = 0;
  module = getApplication().getImportModule (IMPORT_TEST_MOD);
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if ((module != 0) && (datamodel != 0))
    {
      datamodel->detach (this);    // @TODO find another way ?
      module->load("",*datamodel); 
      datamodel->attach (this);
    }
  else
    {
      showError ("Internal Error"); // @TODO make this more clear (?)
    }
  showStatus("Loading done");
  //@TODO improve observer stuff :
  showIndex();
}

void Glade2UI::message_handler(string s)
{
  Glade2UI::myimport_->add_to_output(s);
}

// ------------------------------------------------------------------------
void Glade2UI::loadFile(const char* filename)
{
  GenesImportCore* module = 0;
  //module = getApplication().getImportModule (IMPORT_GED0_MOD);
  module = getApplication().getImportModule (IMPORT_GEDCOM_MOD);
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if ((module != 0) && (datamodel != 0))
    {
      datamodel->block_updates ();
      module->load(filename,*datamodel, Glade2UI::message_handler); 
      datamodel->deblock_updates ();
    }
  else
    {
      showError ("Internal Error"); // @TODO make this more clear (?)
    }
  showStatus("Loading done");
  showIndex();
}

// ------------------------------------------------------------------------
void Glade2UI::showIndex()
{
  genesWnd_->clearRows();

  int nb = 0;
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if (datamodel != 0)
    {
      PersonInfoList& list1 = datamodel->getPersonInfoList ();
      list<PersonInfo*>& list2 = list1.getList ();
      list<PersonInfo*>::iterator iter = list2.begin();
      for (; iter != list2.end(); ++iter)
	{
	  PersonInfo* info = *iter;
	  if (info != 0)
	    {
	      char buffer [20];
	      char buffer2 [20];
	      const char* line[6];
	      const char* ptr1 = GladeUtil::getTextList (info->getSurname());
	      const char* ptr2 = GladeUtil::getTextList (info->getFirstname());
	      int   year = info->getBirthyear();
	      line[0] = ".";
	      line[1] = ptr1;
	      line[2] = ptr2;
	      if (year > 0)
		{
		  sprintf (buffer,"%d",year);
		  line[3] = buffer;
		}
	      else
		{
		  line[3] = "-";
		}
	      year = info->getDeathyear();
	      if (year > 0)
		{
		  sprintf (buffer2,"%d",year);
		  line[4] = buffer2;
		}
	      else
		{
		  line[4] = "-";
		}
	      line[5] = NULL;
	      int row = genesWnd_->addRow(line, (gpointer)info);
	      info->setRow (row);
	    }
	  nb++;
	}
    }
  else
    {
      showError ("internal problem");
    }
}

// ------------------------------------------------------------------------
// There has been an update of the subject that I am observing ...
void Glade2UI::update (Subject*)
{
  cout << "update of indexList needed" << endl;
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if (datamodel != 0)
    {
      datamodel->buildInfoList(); //@TODO has to be updated dynamically (?)
    }
  showIndex();
}

// ------------------------------------------------------------------------
