#ifndef GLADE2UI_H
#define GLADE2UI_H

/***************************************************************************
                            glade2UI.h
                        -------------------
    begin                : 14 Jan 2001
    copyright            : (C) 2000 by The Genes Development Team
    email                : Geert.Vantienen@advalvas.be
    module               : Genes glade2
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <genesCore/genesUICore.h>
#include <glade2/genesComboInfo.h>
#include <datamodel/genesDataId.h>
#include <datamodel/observer.h>
#include <list>

class Family;
class Person;
class Event;
class PersonInfo;
class GenesInfomodel;
class GenesmainWnd;
class ImportWnd;
namespace Gtk
{
 class Main;
}

class Glade2UI : public GenesUICore, public Observer
{
 public:
  Glade2UI (GenesApplication&);
  virtual ~Glade2UI ();

  void load ();
  void view ();
  void about ();
  void selectFather ();
  void selectMother ();
  void selectSpouse (const String&);
  void selectChild (const String&);
  void doOnSelect(int row, int col);
  void doOnDeselect(int row, int col);
  void showPerson ();
  void showFather ();
  void showMother ();
  void showSpouse (const String&);
  void showChild (const String&);
  void addPerson ();

  void gtkinit (int argc, char* argv[]);
  void editEvent (Event&, const String& name);

  virtual void initObject();
  virtual void loop();                          // the main loop
  virtual void quit ();
  virtual void showPerson (Person&);
  virtual void showFamily (Family&);
  void showEvent (Event*, const String& name);

  virtual void update (Subject*);
  virtual void showIndex();

  //virtual void showError (const GenesStatus&);
  virtual void showError (String text);
  virtual void showStatus (String text);
  //virtual void setBusy (int busy);            // @TODO use boolean
  //virtual void askQuestion (String /*, Callback*/); 
  //virtual void createMenus ();
  void        loadTest();
  void        loadFile(const char* filename);
  void        showPerson (const GenesDataId&);

  GenesInfomodel* getDatamodel ();
  
  static void message_handler(string s);
  static ImportWnd* myimport_;

 private:
  Person*     getPerson (const GenesDataId&);
  PersonInfo* getPersonInfo (const GenesDataId&);
  PersonInfo* getFatherInfoFrom (const GenesDataId&);
  PersonInfo* getMotherInfoFrom (const GenesDataId&);
  Family*     getFamily (const GenesDataId&);
  void        clearEntries();
  void        addChildrenTo(Family& fam, list<string>&);
  void        selectPerson (const PersonInfo*);

  PersonInfo*   selectedPersonInfo_;
  GenesDataId   selectedPid_;
  GenesDataId   fatherPid_;
  GenesDataId   motherPid_;
  GenesDataId   spousePid_;
  GenesDataId   childPid_;
  GenesmainWnd* genesWnd_;
  Gtk::Main*    mymain_;
  GenesComboInfo spousesInfo_;
  GenesComboInfo childrenInfo_;

  Glade2UI (const Glade2UI&);  // DO NOT IMPLEMENT
};

#endif
