/***************************************************************************
                           simpleUI.cpp
                        -------------------
    begin                : 16 Dec 2000
    copyright            : (C) 2000 by The Genes Development Team
    email                : Geert.Vantienen@advalvas.be
    module               : Genes Test
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/
// a simple User Interface (only for testing)

#include "simpleUI.h"
#include <genesCore/genesApplication.h>
#include <genesCore/genesImportCore.h>
#include <datamodel/person.h>
#include <datamodel/personInfo.h>
#include <datamodel/personInfoList.h>
#include <datamodel/genesInfomodel.h>
#include <modules.h>
#include <iostream.h>

// ------------------------------------------------------------------------
SimpleUI::SimpleUI (GenesApplication& applic)
  : GenesUICore(applic, GUI_TEST_MOD),
    selectedPid_(GenesDataId::personId)
{
}

// ------------------------------------------------------------------------
SimpleUI::~SimpleUI ()
{
}

// ------------------------------------------------------------------------
// void SimpleUI::initObject ()
// {
//   GenesUICore::initObject(); // future proof ...
// }

// ------------------------------------------------------------------------
void SimpleUI::showMenu ()
{
  GenesImportCore* module 
    = getApplication().getImportModule ("TestImportModule");
  
  cout << "========= Menu ======================" << endl;
  cout << "  m : show menu " << endl;
  cout << "  i : show Person list (index) " << endl;
  cout << "  s : select Person" << endl;
  cout << "  p : show selected person" <<endl;
  if (module != 0)
    {
      cout << "  l : load database from file" << endl;
    }
  cout << "-------------------------------------" << endl;
  cout << "  q : quit" <<endl;
}

// ------------------------------------------------------------------------
void SimpleUI::loop ()
{
  char kar = ' ';
  while (kar != 'q')
    {
      cout << endl << "Enter your choice; [m] for menu" << endl;
      cin >> kar;
      switch (kar)
	{
	case 'm' : showMenu ();
	  break;
	case 'l' : loadTest ();
	  break;
	case 'i' : showIndex ();
	  break;
	case 's' : selectPerson ();
	  break;
	case 'p' : showPerson ();
	  break;
	}
    }
}

// ------------------------------------------------------------------------
void SimpleUI::showError (String text)
{
  cout << "ERROR : " << text << endl;
}

// ------------------------------------------------------------------------
void SimpleUI::showPerson (const Person& person)
{
  cout << "Person (" << person.getId().getId() << ")" <<endl;
  cout << "       FName : " << person.getFirstname() << endl;
  cout << "       SName : " << person.getSurname() << endl;
  //cout << "       Birth: " << person.getBirthyear() << endl;
}

// ------------------------------------------------------------------------
void SimpleUI::showPerson ()
{
  Person* person = getSelectedPerson();
  if (person != 0)
    {
      showPerson (*person);
    }
}

// ------------------------------------------------------------------------
void SimpleUI::selectPerson ()
{
  int id;
  cout << endl << "Enter PID [-1 for none] ";
  cin >> id;
  selectedPid_ = GenesDataId (GenesDataId::personId, id);
}

// ------------------------------------------------------------------------
Person* SimpleUI::getSelectedPerson ()
{
  Person* person = 0;
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if ((selectedPid_.isValid()) && (datamodel != 0))
    {
      person = datamodel->getPerson (selectedPid_);
    }
  return (person);
}

// ------------------------------------------------------------------------
void SimpleUI::loadTest()
{
  GenesImportCore* module 
    = getApplication().getImportModule (IMPORT_GED0_MOD);
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if ((module != 0) && (datamodel != 0))
    {
      module->load("royal.ged",*datamodel); 
    }
  else
    {
      showError ("Internal Error"); // @TODO make this more clear (?)
    }
  cout << "Loading done" << endl;   // @TODO move this to showStatus()
}

// ------------------------------------------------------------------------
void SimpleUI::showIndex()
{
  int nb = 0;
  cout << "==================== index ====================" <<endl;
  cout << "PID   Name             Birthyear " << endl;
  GenesInfomodel* datamodel
    = getApplication().getDatamodel ();
  if (datamodel != 0)
    {
      PersonInfoList& list1 = datamodel->getPersonInfoList ();
      list<PersonInfo*>& list2 = list1.getList ();
      list<PersonInfo*>::iterator iter = list2.begin();
      for (; iter != list2.end(); ++iter)
	{
	  PersonInfo* info = *iter;
	  if (info != 0)
	    {
	      cout << " " << info->getPid().getId() << "  ";
	      cout << info->getFirstname().c_str() << " ";
	      cout << info->getSurname()   << " ";
	      cout << info->getBirthyear() << " ";
	      cout << info->getDeathyear() << endl;
	    }
	  nb++;
	}
    }
  else
    {
      showError ("internal problem");
    }
  cout << "================ total = " << nb << " ===================" <<endl;
}

// ------------------------------------------------------------------------
