/***************************************************************************
                          testImport.cpp
                        -------------------
    begin                : 16 Dec 2000
    copyright            : (C) 2000 by The Genes Development Team
    email                : Geert.Vantienen@advalvas.be
    module               : Genes Test
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "testImport.h"
#include <genesCore/genesApplication.h>
#include <datamodel/genesInfomodel.h>
#include <datamodel/family.h>
#include <datamodel/person.h>
#include <datamodel/note.h>
#include <datamodel/source.h>
#include <datamodel/genesDataId.h>
#include <modules.h>
#include <iostream.h>

// ------------------------------------------------------------------------
TestImport::TestImport (GenesApplication& applic)
  : GenesImportCore (applic, IMPORT_TEST_MOD)
{
}

// ------------------------------------------------------------------------
TestImport::~TestImport ()
{
}

// ------------------------------------------------------------------------
// @TODO later return GenesStatus
void TestImport::load (const String&, GenesInfomodel& datamodel)
{
  cout << "Status: loading file ..." << endl;         // @TODO use showStatus
  // add a few persons to the infomodel.
  // for the sample, this is just a fake one ...

  Person* pers1 = new Person (GenesDataId::getNextPersonId());
  Person* pers2 = new Person (GenesDataId::getNextPersonId());
  Person* pers3 = new Person (GenesDataId::getNextPersonId());
  Person* pers4 = new Person (GenesDataId::getNextPersonId());
  Person* pers5 = new Person (GenesDataId::getNextPersonId());
  Person* pers6 = new Person (GenesDataId::getNextPersonId());
  Person* pers7 = new Person (GenesDataId::getNextPersonId());
  Family* fam1  = new Family (GenesDataId::getNextFamilyId());
  Family* fam2  = new Family (GenesDataId::getNextFamilyId());
  Family* fam3  = new Family (GenesDataId::getNextFamilyId());
  Source* src1  = new Source (GenesDataId::getNextSourceId());
  Note*   note1 = new Note (GenesDataId::getNextNoteId());

  pers1->setFirstname ("Elizabeth Alexandra Mary");
  pers1->setSurname   ("Windsor");
  pers1->setOriginalId("@I0001@");
  pers2->setFirstname ("Philip");
  pers2->setSurname   ("Mountbaten");
  pers2->setOriginalId("@I0002@");
  pers3->setFirstname ("Charles Philip Arthur George");
  pers3->setSurname   ("Windsor");
  pers3->setOriginalId("@I0006@");
  pers4->setFirstname ("Diana");
  pers4->setSurname   ("Spencer");
  pers4->setOriginalId("@I0010@");
  pers5->setFirstname ("Camila");
  pers5->setSurname   ("Parker-Bowles");
  pers6->setFirstname ("William");
  pers6->setSurname   ("Windsor");
  pers7->setFirstname ("Harry");
  pers7->setSurname   ("Windsor");
  pers7->setOriginalId("@I0012@");

  Event* birth1 = new Event (GenesDataId::getNextEventId(),Event::birth);
  Event* birth2 = new Event (GenesDataId::getNextEventId(),Event::birth);
  Event* birth3 = new Event (GenesDataId::getNextEventId(),Event::birth);
  Event* birth4 = new Event (GenesDataId::getNextEventId(),Event::birth);
  Event* birth5 = new Event (GenesDataId::getNextEventId(),Event::birth);
  Event* death4 = new Event (GenesDataId::getNextEventId(),Event::death);
  Event* marriage1 = new Event (GenesDataId::getNextEventId(),Event::marriage);
  Event* marriage2 = new Event (GenesDataId::getNextEventId(),Event::marriage);
  Event* divorce = new Event (GenesDataId::getNextEventId(),Event::divorce);

  birth1->setDate (21,4,1926);
  birth1->setPlace ("London");
  birth2->setDate (0,0,1921);
  birth2->setPlace ("");
  birth3->setDate (14,11,1948);
  birth3->setPlace ("");
  birth4->setDate (0,7,1961);
  birth4->setPlace ("");
  death4->setDate (31,8,1997);
  death4->setPlace ("Paris");

  marriage1->setDate (20,11,1947);
  marriage1->setPlace ("London");
  marriage2->setDate (28,7,1981);
  marriage2->setPlace ("London ???");
  divorce->setDate (9,8,2001);
  divorce->setPlace ("never");
  
  src1->setTitle ("Based on royal.ged");
  src1->setAuthor ("ftree");
  src1->addText ("Some of this data is fake");
  src1->addText ("It is only meant for testing");

  note1->addText ("This is fake information !");
  //divorce->addNote (note1->getId());

  pers1->addEvent (birth1);
  pers2->addEvent (birth2);
  pers3->addEvent (birth3);
  pers4->addEvent (birth4);
  pers4->addEvent (death4);
  pers5->addEvent (birth5);

  fam1->setFather (pers2->getId());
  fam1->setMother (pers1->getId());
  fam1->addChild (pers3->getId());
  fam1->addEvent (marriage1);
  fam2->setFather (pers3->getId());
  fam2->setMother (pers4->getId());
  fam2->addChild (pers6->getId());
  fam2->addChild (pers7->getId());
  fam2->addEvent (marriage2);
  fam2->addEvent (divorce);
  fam3->setFather (pers3->getId());
  fam3->setMother (pers5->getId()); // only for testing multiple spouses
  pers1->addSpouseFamily (fam1->getId());
  pers2->addSpouseFamily (fam1->getId());
  pers3->setChildFamily (fam1->getId());
  pers3->addSpouseFamily (fam2->getId());
  pers4->addSpouseFamily (fam2->getId());
  pers6->setChildFamily  (fam2->getId());
  pers7->setChildFamily  (fam2->getId());
  pers3->addSpouseFamily (fam3->getId());
  pers5->addSpouseFamily (fam3->getId());

  datamodel.addPerson (pers1);
  datamodel.addPerson (pers2);
  datamodel.addPerson (pers3);
  datamodel.addPerson (pers4);
  datamodel.addPerson (pers5);
  datamodel.addPerson (pers6);
  datamodel.addPerson (pers7);
  datamodel.addFamily (fam1);
  datamodel.addFamily (fam2);
  datamodel.addFamily (fam3);
  datamodel.addSource (src1);
  datamodel.addNote (note1);

  // @TODO notify ui that loading is done ...
  datamodel.buildInfoList();
}


// ------------------------------------------------------------------------
