/***************************************************************************
                             showUtils.cpp 
                             -------------------
    begin                : Fri Mar 15 2001
    copyright            : (C) 2001 by The Genes Development Team
    email                : Geert.Vantienen@advalvas.be
    module               : Genes Utils
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <utils/showUtils.h>
#include <datamodel/genesInfomodel.h>
#include <datamodel/personInfo.h>
#include <datamodel/family.h>
#include <datamodel/event.h>
#include <stdio.h>
#include <libintl.h>
#define _(string) gettext (string)
#define N_(string) (string)

// show some information depending on the users preferences ...

// ------------------------------------------------------------------------
String ShowUtils::getDateString (const Event& event)
{
  char buffer [50];
  if (event.isValid())
    {
      sprintf (buffer,"%02d-%02d-%04d",event.getDay(), event.getMonth(),
	       event.getYear());
      return (String (buffer));
    }
  else
    {
      return (String(""));
    }
}

// ------------------------------------------------------------------------
String ShowUtils::getAgeString (const Event* birth, const Event* event)
{
  char buffer [50];
  if (event == 0)
    {
      return ("NULL"); // should never happen ?
    }
  if (birth == 0)
    {
      return ("?");
    }
  if (event->getEventType() == Event::birth)
    {
      return (String ("-"));
    }
  else if (event->isValid())
    {
      int y = event->getYear() - birth->getYear();
      //int m = event->getMonth() - birth->getMonth();
      //int d = event->getDay() - birth->getDay();
      if ((y >= 0) && (birth->getYear() > 0))
	{
	  sprintf (buffer,"%d y", y);
	}
      else
	{
	  sprintf (buffer, "?");
	}
      return (String (buffer));
    }
  else
    {
      return (String(""));
    }
}

// ------------------------------------------------------------------------
String ShowUtils::getFullName (const Person& person)
{
  const String& surname = person.getSurname();
  const String& firstname = person.getFirstname();
  String name = firstname + " " + surname;
  return (name);
}

// ------------------------------------------------------------------------
String ShowUtils::getFullName2 (const PersonInfo& person)
{
  const String& surname = person.getSurname();
  const String& firstname = person.getFirstname();
  String name = firstname + " " + surname;
  return (name);
}

// ------------------------------------------------------------------------
String ShowUtils::getFamilyName (GenesInfomodel& datamodel,
				 const Family& family)
{
  String famName = "";
  String name1 = "";
  String name2 = "";
  const GenesDataId& id1 = family.getFather();
  const GenesDataId& id2 = family.getMother();
  Person* father = datamodel.getPerson (id1);
  if (father != 0)
    {
      name1 = father->getSurname();
    }
  Person* mother = datamodel.getPerson (id2);
  if (mother != 0)
    {
      name2 = mother->getSurname();
    }
  if ((name1 == "") || (name2 == ""))
    {
      famName = name1 + name2;
    }
  else
    {
      famName = name1 + " - " + name2;
    }
  return (famName);
}

// ------------------------------------------------------------------------
String ShowUtils::getEventName (Event::EventType eventType)
{
  String name = "";
  switch (eventType)
    {
    case Event::undefined :
      name = _("Unknown");
      break;
    case Event::birth :
      name = _("Birth");
      break;
    case Event::christening :
      name = _("Christening");
      break;
    case Event::baptism :
      name = _("Baptism");
      break;
    case Event::bar_mitzvah :
      name = _("Bar mitzvah");
      break;
    case Event::first_communion :
      name = _("First communion");
      break;
    case Event::graduation :
      name = _("Graduation");
      break;
    case Event::naturalization :
      name = _("Naturalization");
      break;
    case Event::emigration :
      name = _("Emigration");
      break;
    case Event::immigration :
      name = _("Immigration");
      break;
    case Event::engagement :
      name = _("Engagement");
      break;
    case Event::marriage :
      name = _("Marriage");
      break;
    case Event::census :
      name = _("Census");
      break;
    case Event::retirement :
      name = _("Retirement");
      break;
    case Event::divorce :
      name = _("Divorce");
      break;
    case Event::divorce_filed :
      name = _("Divorce filed");
      break;
    case Event::death :
      name = _("Death");
      break;
    case Event::buriel :
      name = _("Buriel");
      break;
    case Event::cremation :
      name = _("Cremation");
      break;
    case Event::other :
      name = _("Unknown");
      break;
    }
  return (name);
}

// ------------------------------------------------------------------------
String ShowUtils::getAttributeName (Attribute::AttributeType attrType)
{
  String name = "";
  switch (attrType)
    {
    case Attribute::undefined :
      name = _("Unknown");
      break;
    case Attribute::title :
      name = _("Title");
      break;
    case Attribute::education :
      name = _("Education");
      break;
    case Attribute::occupation :
      name = _("Occupation");
      break;
    case Attribute::religion :
      name = _("Religion");
      break;
    case Attribute::caste :
      name = _("Caste");
      break;
    case Attribute::phy_descr :
      name = _("Physical description");
      break;
    case Attribute::ident_no :
      name = _("Ident number");
      break;
    case Attribute::nationality :
      name = _("Nationality");
      break;
    case Attribute::child_count :
      name = _("Number of children");
      break;
    case Attribute::marriage_count :
      name = _("Number of marriages");
      break;
    case Attribute::property :
      name = _("Property");
      break;
    case Attribute::soc_sec :
      name = _("Social security number");
      break;
    }
  return (name);
}

// ------------------------------------------------------------------------
String ShowUtils::getSexString (Person::SexType sex )
{
  String sexStr = "";
  if (sex == Person::female)
    {
      sexStr = _("female");
    }
  else if (sex == Person::male)
    {
      sexStr = _("male");
    }
  else
    {
      sexStr = "?";
    }
  return (sexStr);
}

// ------------------------------------------------------------------------
